@@ -1,25 +1,23 @@
 import datetime as dt
 import logging
-from zoneinfo import ZoneInfo
+from typing import Literal
 
 import pandas as pd
+import polars as pl
 
-import pyield.date_converter as dc
+import pyield.converters as cv
 from pyield import b3, bday, interpolator
+from pyield.config import TIMEZONE_BZ
+from pyield.converters import DateArray, DateScalar
 from pyield.data_cache import get_cached_dataset
-from pyield.date_converter import DateArray, DateScalar
 
 logger = logging.getLogger(__name__)
 
-TIMEZONE_BZ = ZoneInfo("America/Sao_Paulo")
 
-
-def _get_data(dates: pd.Series) -> pd.DataFrame:
+def _load_with_intraday(dates: list[dt.date]) -> pl.DataFrame:
     """Busca dados de DI, incluindo dados intraday para o dia corrente se necessário."""
     # 1. Busca inicial no cache com as datas solicitadas pelo usuário.
-    df_cached = (
-        get_cached_dataset("di1").query("TradeDate in @dates").reset_index(drop=True)
-    )
+    df_cached = get_cached_dataset("di1").filter(pl.col("TradeDate").is_in(dates))
 
     today = dt.datetime.now(TIMEZONE_BZ).date()
     last_bday = bday.last_business_day()
@@ -27,27 +25,28 @@ def _get_data(dates: pd.Series) -> pd.DataFrame:
     # 2. Lógica para buscar dados intraday.
     #    Isso é necessário quando o usuário solicita os dados do dia corrente
     #    e eles ainda não foram persistidos no cache (processo noturno).
-    if (
-        # Condição 1: O dia de hoje foi explicitamente solicitado.
-        today in dates.values
-        # Condição 2: E ainda não está no cache.
-        and today not in df_cached["TradeDate"].values
-        # Condição 3: E hoje é o último dia útil.
-        and today == last_bday
-    ):
+    # Condição 1: O dia de hoje foi solicitado.
+    has_today = today in dates
+    # Condição 2: E ainda não está no cache.
+    is_today_not_in_cache = df_cached.filter(pl.col("TradeDate") == today).is_empty()
+    # Condição 3: E hoje é o último dia útil.
+    is_today_last_bday = today == last_bday
+
+    if has_today and is_today_not_in_cache and is_today_last_bday:
         try:
             df_intraday = b3.futures(contract_code="DI1", date=today)
-            if "SettlementPrice" not in df_intraday.columns or df_intraday.empty:
+            df_intraday = pl.from_pandas(df_intraday)
+            if "SettlementPrice" not in df_intraday.columns or df_intraday.is_empty():
                 logger.warning(
                     f"Ainda sem dados de ajustes intraday para {today}."
                     " Retornando apenas dados do cache."
                 )
+                df_intraday = df_intraday.drop("DaysToExp", strict=False)
+
                 return df_cached
-            if "DaysToExp" in df_intraday.columns:
-                df_intraday.drop(columns=["DaysToExp"], inplace=True)
-            if df_cached.empty:
-                return df_intraday.reset_index(drop=True)
-            return pd.concat([df_cached, df_intraday]).reset_index(drop=True)
+            if df_cached.is_empty():
+                return df_intraday
+            return pl.concat([df_cached, df_intraday], how="diagonal")
         except Exception as e:
             logger.error(f"Falha ao buscar dados intraday para {today}: {e}")
 
@@ -55,12 +54,31 @@ def _get_data(dates: pd.Series) -> pd.DataFrame:
     return df_cached
 
 
+def _get_data(dates: DateScalar | DateArray) -> pl.DataFrame:
+    converted_dates = cv.convert_input_dates(dates)
+
+    match converted_dates:
+        case None:
+            logger.warning("No valid dates provided. Returning empty DataFrame.")
+            return pl.DataFrame()
+        case dt.date():
+            dates_list = [converted_dates]
+        case pd.Series():
+            dates_list = (
+                pl.Series("TradeDate", converted_dates).unique().sort().to_list()
+            )
+
+    df = _load_with_intraday(dates_list)
+
+    return df.sort(by=["TradeDate", "ExpirationDate"])
+
+
 def data(
     dates: DateScalar | DateArray,
     month_start: bool = False,
     pre_filter: bool = False,
-    all_columns: bool = True,
-) -> pd.DataFrame:
+    return_format: Literal["pandas", "polars"] = "pandas",
+) -> pd.DataFrame | pl.DataFrame:
     """
     Retrieves DI Futures contract data for a specific trade date.
 
@@ -76,13 +94,12 @@ def data(
             those whose expiration dates match known prefixed Treasury bond (LTN, NTN-F)
             maturities from the TPF dataset nearest to the given trade date.
             Defaults to False.
-        all_columns (bool, optional): If True, returns all available columns from
-            the DI dataset. If False, returns a subset of the most common columns.
-            Defaults to True.
+        return_format (Literal['pandas','polars'], optional): Specifies the return
+            format. Can be 'pandas' (default) or 'polars'.
 
     Returns:
-        pd.DataFrame: A DataFrame containing the DI futures contract data for the
-            specified dates, sorted by trade dates and expiration dates.
+        pd.DataFrame | pl.DataFrame: A DataFrame containing the DI futures contract
+            data for the specified dates, sorted by trade dates and expiration dates.
             Returns an empty DataFrame if no data is found
 
     Examples:
@@ -96,93 +113,90 @@ def data(
         3 2024-10-16     2025-02-01       DI1G25          74         279491
         4 2024-10-16     2025-03-01       DI1H25          94         344056
     """
-    dates = dc.convert_input_dates(dates)
-    # Force dates to be a Series for consistency
-    if isinstance(dates, dt.date):
-        dates = pd.Series([dates]).astype("date32[pyarrow]")
+    df = _get_data(dates=dates)
 
-    # Remove duplicates and sort dates
-    dates = dates.drop_duplicates().sort_values().reset_index(drop=True)
-    df = _get_data(dates)
-    if df.empty:
-        logger.warning("No DI1 data found for the specified dates.")
-        logger.warning("Returning empty DataFrame.")
-        return pd.DataFrame()
-
-    if "DaysToExpiration" in df.columns:
-        df.drop(columns=["DaysToExpiration"], inplace=True)
+    if month_start:
+        df = df.with_columns(pl.col("ExpirationDate").dt.truncate("1mo"))
 
     if pre_filter:
         df_pre = (
             get_cached_dataset("tpf")
-            .query("BondType in ['LTN', 'NTN-F']")
-            .drop_duplicates(subset=["ReferenceDate", "MaturityDate"])
-            .sort_values(["ReferenceDate", "MaturityDate"])
-            .reset_index(drop=True)[["ReferenceDate", "MaturityDate"]]
-            .rename(  # Rename columns for merging
-                columns={"ReferenceDate": "TradeDate", "MaturityDate": "ExpirationDate"}
+            .filter(pl.col("BondType").is_in(["LTN", "NTN-F"]))
+            .unique(subset=["ReferenceDate", "MaturityDate"])
+            .select(
+                pl.col("ReferenceDate").alias("TradeDate"),
+                pl.col("MaturityDate").alias("ExpirationDate"),
             )
         )
 
-        # Verificar se existe a data de hoje no df_pre
-        today = dt.datetime.now(TIMEZONE_BZ).date()
-        if today in dates.values and today not in df_pre["TradeDate"].values:
-            df_pre_today = df_pre.query("TradeDate == TradeDate.max()").reset_index(
-                drop=True
-            )
-            df_pre_today["TradeDate"] = today
-            df_pre_today["TradeDate"] = df_pre_today["TradeDate"].astype(
-                "date32[pyarrow]"
-            )
-            df_pre = pd.concat([df_pre, df_pre_today]).reset_index(drop=True)
+        # garante que os dois lados estão ordenados pelas chaves necessárias
+        df = df.sort(["TradeDate", "ExpirationDate"])
+        df_pre = df_pre.sort(["TradeDate", "ExpirationDate"])
 
-        # Assure that dates in ExpirationDate (maturity date) are business days
-        df_pre["ExpirationDate"] = bday.offset(df_pre["ExpirationDate"], 0).astype(
-            "date32[pyarrow]"
+        df = df.join_asof(
+            df_pre,
+            left_on="TradeDate",
+            right_on="TradeDate",
+            by="ExpirationDate",  # garante matching por vértice
+            strategy="backward",  # pega a data anterior se não tiver exata
+            check_sortedness=False,  # já garantimos a ordenação
         )
 
-        df = df.merge(df_pre, how="inner", on=["TradeDate", "ExpirationDate"])
+    return cv.to_return_format(df, return_format)
 
-        if df.empty:
-            logger.warning(
-                "No DI Futures data found for the specified dates after pre-filtering."
-                "Returning empty DataFrame."
-            )
-            return pd.DataFrame()
 
-    if month_start:
-        df["ExpirationDate"] = df["ExpirationDate"].dt.floor("MS")
+def _build_input_dataframe(
+    dates: DateScalar | DateArray,
+    expirations: DateScalar | DateArray,
+) -> pl.DataFrame:
+    # 1. Converte as entradas primeiro
+    converted_dates = cv.convert_input_dates(dates)
+    converted_expirations = cv.convert_input_dates(expirations)
 
-    if not all_columns:
-        cols = [
-            "TradeDate",
-            "TickerSymbol",
-            "ExpirationDate",
-            "BDaysToExp",
-            "OpenContracts",
-            "TradeVolume",
-            "DV01",
-            "SettlementPrice",
-            "LastPrice",
-            "OpenRate",
-            "MinRate",
-            "MaxRate",
-            "CloseRate",
-            "SettlementRate",
-            "LastRate",
-            "ForwardRate",
-        ]
-        selected_cols = [col for col in cols if col in df.columns]
-        df = df[selected_cols].copy()
+    # 2. Lida com os 4 casos de forma SIMPLES E LEGÍVEL
+    match (converted_dates, converted_expirations):
+        # CASO 1: Data escalar, vencimentos em array
+        case dt.date() as d, pd.Series() as e:
+            if e.empty:
+                dfi = pl.DataFrame()
+            else:
+                # Cria o DF com o array, e ADICIONA o escalar com pl.lit()
+                dfi = pl.DataFrame({"ExpirationDate": e}).with_columns(TradeDate=d)
 
-    return df.sort_values(by=["TradeDate", "ExpirationDate"]).reset_index(drop=True)
+        # CASO 2: Datas em array, vencimento escalar
+        case pd.Series() as d, dt.date() as e:
+            if d.empty:
+                dfi = pl.DataFrame()
+            else:
+                # Mesma lógica, invertida
+                dfi = pl.DataFrame({"TradeDate": d}).with_columns(ExpirationDate=e)
+
+        # CASO 3: Ambos são arrays
+        case pd.Series() as d, pd.Series() as e:
+            if d.empty or e.empty:
+                dfi = pl.DataFrame()
+            elif len(d) != len(e):
+                raise ValueError("'dates' e 'expirations' devem ter o mesmo tamanho.")
+            else:
+                dfi = pl.DataFrame({"TradeDate": d, "ExpirationDate": e})
+
+        # CASO 4: Ambos são escalares
+        case dt.date() as d, dt.date() as e:
+            dfi = pl.DataFrame({"TradeDate": [d], "ExpirationDate": [e]})
+
+        # QUALQUER OUTRA COISA
+        case _:
+            dfi = pl.DataFrame()
+
+    return dfi
 
 
 def interpolate_rates(
-    dates: DateScalar | DateArray,
-    expirations: DateScalar | DateArray,
+    dates: DateScalar | DateArray | None,
+    expirations: DateScalar | DateArray | None,
     extrapolate: bool = True,
-) -> pd.Series:
+    return_format: Literal["pandas", "polars"] = "pandas",
+) -> pd.Series | pl.Series:
     """
     Interpolates DI rates for specified trade dates and expiration dates.
 
@@ -229,7 +243,7 @@ def interpolate_rates(
         0    0.13972
         1    0.134613
         2    <NA>
-        dtype: double[pyarrow]
+        Name: irate, dtype: double[pyarrow]
 
         >>> # Interpolate rates for a single trade date and multiple expiration dates
         >>> # There is no DI Contract in 09-05-2025 with expiration 01-01-2050
@@ -241,7 +255,7 @@ def interpolate_rates(
         ... )
         0    0.13901
         1    0.13881
-        dtype: double[pyarrow]
+        Name: irate, dtype: double[pyarrow]
 
         >>> # With extrapolation set to False, the second rate will be NaN
         >>> # Note: 0.13576348733268917 is shown as 0.135763
@@ -251,56 +265,38 @@ def interpolate_rates(
         ...     extrapolate=False,
         ... )
         0    0.135763
-        1    <NA>
-        dtype: double[pyarrow]
+        1    NaN
+        Name: irate, dtype: double[pyarrow]
 
     Notes:
         - All available settlement rates are used for the flat-forward interpolation.
         - The function handles broadcasting of scalar and array-like inputs.
     """
-    # Convert input dates to a consistent format
-    dates = dc.convert_input_dates(dates)
-    expirations = dc.convert_input_dates(expirations)
+    dfi = _build_input_dataframe(dates, expirations)
 
-    # Ensure the lengths of input arrays are consistent
-    match (dates, expirations):
-        case dt.date(), pd.Series():
-            dfi = pd.DataFrame({"mat": expirations})
-            dfi["tdate"] = dates
+    # 2. Se a helper retornou None, a entrada é inválida.
+    if dfi.is_empty():
+        logger.warning("Invalid or empty dates provided. Returning empty Series.")
+        return pd.Series(dtype="float64[pyarrow]")
 
-        case pd.Series(), dt.date():
-            dfi = pd.DataFrame({"tdate": dates})
-            dfi["mat"] = expirations
-
-        case pd.Series(), pd.Series():
-            if len(dates) != len(expirations):
-                raise ValueError("Args. should have the same length.")
-            dfi = pd.DataFrame({"tdate": dates, "mat": expirations})
-
-        case dt.date(), dt.date():
-            dfi = pd.DataFrame({"tdate": [dates], "mat": [expirations]})
-
-    # Compute business days between reference dates and maturities
-    dfi["bdays"] = bday.count(dfi["tdate"], dfi["mat"])
-
-    # Initialize the interpolated rate column with NaN
-    dfi["irate"] = pd.NA
-    dfi["irate"] = dfi["irate"].astype("float64[pyarrow]")
+    # bday.count retorna uma Series de inteiros do pandas
+    s_bdays = bday.count(dfi["TradeDate"], dfi["ExpirationDate"])
+    dfi = dfi.with_columns(pl.Series("bdays", s_bdays), irate=None)
 
     # Load DI rates dataset filtered by the provided reference dates
-    dfr = data(dates=dates)
+    dfr = _get_data(dates=dates)
 
     # Return an empty DataFrame if no rates are found
-    if dfr.empty:
-        return pd.Series()
+    if dfr.is_empty():
+        return pd.Series(dtype="float64[pyarrow]")
 
     # Iterate over each unique reference date
-    for date in dfi["tdate"].unique():
+    for date in dfi.get_column("TradeDate").unique().to_list():
         # Filter DI rates for the current reference date
-        dfr_subset = dfr.query("TradeDate == @date").reset_index(drop=True)
+        dfr_subset = dfr.filter(pl.col("TradeDate") == date)
 
         # Skip processing if no rates are available for the current date
-        if dfr_subset.empty:
+        if dfr_subset.is_empty():
             continue
 
         # Initialize the interpolator with known rates and business days
@@ -311,13 +307,16 @@ def interpolate_rates(
             extrapolate=extrapolate,
         )
 
-        # Apply interpolation to rows matching the current reference date
-        mask: pd.Series = dfi["tdate"] == date
-        dfi.loc[mask, "irate"] = dfi.loc[mask, "bdays"].apply(interp)
+        dfi = dfi.with_columns(
+            pl.when(pl.col("TradeDate") == date)
+            .then(pl.col("bdays").map_elements(interp, return_dtype=pl.Float64))
+            .otherwise(pl.col("irate"))
+            .alias("irate")
+        )
 
-    # Return the Series with interpolated rates
-    dfi["irate"].name = None
-    return dfi["irate"]
+    # Return the series with interpolated rates
+    irates = dfi.get_column("irate")
+    return cv.to_return_format(irates, return_format)
 
 
 def interpolate_rate(
@@ -362,47 +361,60 @@ def interpolate_rate(
         >>> di1.interpolate_rate("25-04-2025", "01-01-2050", extrapolate=True)
         0.13881
     """
-    date = dc.convert_input_dates(date)
-    expiration = dc.convert_input_dates(expiration)
-    if not date or not expiration:
-        return float("NaN")
+    converted_date = cv.convert_input_dates(date)
+    converted_expiration = cv.convert_input_dates(expiration)
+
+    if not isinstance(converted_date, dt.date) or not isinstance(
+        converted_expiration, dt.date
+    ):
+        raise ValueError("Both 'date' and 'expiration' must be single date values.")
+
+    if not converted_date or not converted_expiration:
+        return float("nan")
 
     # Get the DI contract DataFrame
-    df = data(dates=date)
+    df = _get_data(dates=converted_date)
 
-    if df.empty:
-        return float("NaN")
+    if df.is_empty():
+        return float("nan")
 
-    max_exp = df["ExpirationDate"].max()
+    max_exp = df.get_column("ExpirationDate").max()
 
-    if expiration > max_exp and not extrapolate:
+    if converted_expiration > max_exp and not extrapolate:
         logger.warning(
-            f"Expiration date ({expiration}) is greater than the maximum expiration "
+            f"Expiration ({converted_expiration}) is greater than the maximum exp. "
             f"date ({max_exp}) and extrapolation is not allowed. Returning NaN."
         )
-        return float("NaN")
+        return float("nan")
 
-    if expiration in df["ExpirationDate"]:
-        rate = df.query("ExpirationDate == @expiration")["SettlementRate"]
-        return float(rate.iloc[0]) if not rate.empty else float("NaN")
+    rate = df.filter(pl.col("ExpirationDate") == converted_expiration).get_column(
+        "SettlementRate"
+    )
+
+    if not rate.is_empty():
+        logger.info(f"Exact match found for expiration {converted_expiration}.")
+        return rate.item(0)
 
     ff_interp = interpolator.Interpolator(
         method="flat_forward",
-        known_bdays=df["BDaysToExp"],
-        known_rates=df["SettlementRate"],
+        known_bdays=df.get_column("BDaysToExp"),
+        known_rates=df.get_column("SettlementRate"),
         extrapolate=extrapolate,
     )
-    bd = bday.count(date, expiration)
-    if pd.isna(bd):
-        return float("NaN")
+    bd = bday.count(converted_date, converted_expiration)
+    if not bd:
+        return float("nan")
+
     return ff_interp(bd)
 
 
-def available_trade_dates() -> pd.Series:
+def available_trade_dates(
+    return_format: Literal["pandas", "polars"] = "pandas",
+) -> pd.Series | pl.Series:
     """
-    Returns all unique end-of-day trade dates available in the DI dataset.
+    Returns all available (completed) trading dates in the DI dataset.
 
-    Retrieves and lists all distinct 'TradeDate' values present in the
+    Retrieves distinct 'TradeDate' values present in the
     historical DI futures data cache, sorted chronologically.
 
     Returns:
@@ -418,13 +430,13 @@ def available_trade_dates() -> pd.Series:
         2   1995-01-04
         3   1995-01-05
         4   1995-01-06
-        dtype: date32[day][pyarrow]
+        Name: available_dates, dtype: date32[day][pyarrow]
     """
     available_dates = (
         get_cached_dataset("di1")
-        .drop_duplicates(subset=["TradeDate"])["TradeDate"]
-        .sort_values(ascending=True)
-        .reset_index(drop=True)
+        .get_column("TradeDate")
+        .unique()
+        .sort()
+        .alias("available_dates")
     )
-    available_dates.name = None
-    return available_dates
+    return cv.to_return_format(available_dates, return_format)
