@@ -5,12 +5,12 @@ from zoneinfo import ZoneInfo
 
 import pandas as pd
 
+import pyield.converters as cv
 from pyield import bday
-from pyield import date_converter as dc
 from pyield.b3.futures.historical import fetch_bmf_data
 from pyield.b3.futures.intraday import fetch_intraday_df
 from pyield.b3.futures.xml import fetch_xml_data
-from pyield.date_converter import DateScalar
+from pyield.converters import DateScalar
 
 BZ_TIMEZONE = ZoneInfo("America/Sao_Paulo")
 ContractOptions = Literal["DI1", "DDI", "FRC", "DAP", "DOL", "WDO", "IND", "WIN"]
@@ -108,7 +108,7 @@ def futures(
         3  2024-05-31       DAPU24  ...         0.0855     0.078171
         ...
     """
-    converted_date = dc.convert_input_dates(date)
+    converted_date = cv.convert_input_dates(date)
     selected_contract = str(contract_code).upper()
 
     if _is_trading_day(converted_date):
