@@ -25,10 +25,10 @@ import polars as pl
 import polars.selectors as cs
 import requests
 
+import pyield.bc.ptax_api as pt
+import pyield.converters as cv
 from pyield import bday
-from pyield import date_converter as dc
-from pyield.bc import ptax_api as pt
-from pyield.date_converter import DateScalar
+from pyield.converters import DateScalar
 from pyield.retry import default_retry
 from pyield.tn.ntnb import duration as duration_b
 from pyield.tn.ntnf import duration as duration_f
@@ -95,12 +95,12 @@ def _build_url(
 ) -> str:
     url = BASE_API_URL
     if start:
-        start = dc.convert_input_dates(start)
+        start = cv.convert_input_dates(start)
         start_str = start.strftime("%Y-%m-%d")
         url += f"@dataMovimentoInicio='{start_str}'"
 
     if end:
-        end = dc.convert_input_dates(end)
+        end = cv.convert_input_dates(end)
         end_str = end.strftime("%Y-%m-%d")
         url += f"&@dataMovimentoFim='{end_str}'"
 
@@ -234,7 +234,7 @@ def _adjust_values_without_auction(df: pl.DataFrame) -> pl.DataFrame:
     return df
 
 
-def _add_duration(df: pl.DataFrame) -> pd.DataFrame:
+def _add_duration(df: pl.DataFrame) -> pl.DataFrame:
     """
     Calcula a duration para cada tipo de título, aplicando uma função
     linha a linha para os casos não-vetorizáveis (NTN-F e NTN-B).
@@ -277,17 +277,6 @@ def _add_dv01(df: pl.DataFrame) -> pl.DataFrame:
         (dv01_unit_expr * pl.col("AcceptedQuantity")).alias("DV01"),
         (dv01_unit_expr * pl.col("AcceptedQuantityFR")).alias("DV01FR"),
         (dv01_unit_expr * pl.col("AcceptedQuantitySR")).alias("DV01SR"),
-    ).with_columns(
-        # 3. Aplica a limpeza final:
-        #    - Se for LFT, o valor é 0.0
-        #    - Senão, usa o valor calculado.
-        #    - Preenche qualquer nulo restante com 0.
-        #    - Arredonda para 2 casas decimais.
-        pl.when(pl.col("BondType") == "LFT")
-        .then(0.0)
-        .otherwise(pl.col("DV01", "DV01FR", "DV01SR"))
-        .fill_null(0)
-        .name.keep()  # Mantém os nomes originais das colunas
     )
 
     return df
@@ -335,7 +324,7 @@ def _add_usd_dv01(df: pl.DataFrame) -> pl.DataFrame:
 
     df = (
         df.sort("Date")  # Importante para o join_asof
-        .join_asof(df_ptax, on="Date", strategy="forward")
+        .join_asof(df_ptax, on="Date", strategy="backward")
         .with_columns(
             (cs.starts_with("DV01") / pl.col("PTAX")).round(2).name.suffix("USD")
         )
